#!/usr/bin/python
# -*- coding: UTF-8 -*-

from cStringIO import StringIO
from flask import *
from flask_compress import Compress
from threading import Thread
from werkzeug import *

import hashlib
import logging
import os
import tempfile
import zipfile

from MDmisc.egit import git_version
from MDmisc.RecursiveDefaultDict import defDict

from functions import *

################################################################################

logging.getLogger().setLevel( logging.INFO )

################################################################################

api = Flask( __name__ )
Compress( api )

UPLOAD_FOLDER = os.path.join( tempfile.gettempdir(), "NIST" )
api.config[ 'UPLOAD_FOLDER' ] = UPLOAD_FOLDER

ALLOWED_EXTENSIONS = [ '.nist', '.nst', '.an2', '.xml' ]

################################################################################
#    Generic routing

@api.route( '/ping' )
def ping():
    return "pong"

@api.route( '/version' )
def version():
    return jsonify( git_version() )

################################################################################
#    Generic file serving

@api.route( '/cdn/<directory>/<subpath>' )
def send_cdn( directory, subpath ):
    if not directory in [ 'js', 'css' ]:
        abort( 403 )
    else:
        return send_from_directory( "./cdn/" + directory, subpath )

@api.route( '/app/<path>' )
def send_app_files( path ):
    return send_from_directory( 'app', path )

################################################################################
#    File upload

@api.route( '/upload', methods = [ 'GET', 'POST' ] )
def upload_file():
    if request.method == 'POST':
        if 'file' not in request.files:
            return jsonify( { 'error': True, 'message': 'No file in the POST request' } )
        
        file = request.files[ 'file' ]
        
        if file.filename == '':
            return jsonify( { 'error': True, 'message': 'No file selected' } )
        
        _, ext = os.path.splitext( file.filename )
        if file and ext in ALLOWED_EXTENSIONS:
            filename = secure_filename( file.filename )
            
            file.save( os.path.join( UPLOAD_FOLDER, filename ) )
            n = NISTf_auto( UPLOAD_FOLDER + "/" + filename )
            
            Thread( target = process, args = ( UPLOAD_FOLDER, filename ) ).start()
            
            return jsonify( { 'error': False, 'file': filename, 'hash': n.hash() } )
        
        else:
            return jsonify( { 'error': True, 'message': 'Format not allowed' } )

@api.route( '/info/<h>/<filename>' )
def get_info( h, filename ):
    if os.path.isfile( UPLOAD_FOLDER + "/" + h + "/" + filename ):
        n = NISTf_auto( UPLOAD_FOLDER + "/" + h + "/" + filename )
        
        ntypes = n.get_ntype()
        sizes = defDict()
        
        for ntype in ntypes:
            for idc in n.get_idc( ntype ):
                try:
                    fpc = n.get_field( ( ntype, fpcfield[ ntype ] ), idc )
                    sizes[ ntype ][ fpc ] = n.get_size( idc = idc )
                
                except:
                    continue
        
        return jsonify( {
            'error': False,
            'data': {
                'ntypes': ntypes,
                'sizes': sizes,
            }
        } )
    
    else:
        return jsonify( {
           'error': True,
           'message': 'File not found' 
        } ) 

@api.route( '/dump/<h>/<filename>' )
def get_txt_dump( h, filename ):
    if os.path.isfile( UPLOAD_FOLDER + "/" + h + "/" + filename ):
        n = NISTf_auto( UPLOAD_FOLDER + "/" + h + "/" + filename )
        
        return jsonify( {
            'error': False,
            'data': str( n )
        } )
    
    else:
        return jsonify( {
           'error': True,
           'message': 'File not found' 
        } )

@api.route( '/img/<h>/<filename>' )
def get_file( h, filename ):
    if os.path.isfile( UPLOAD_FOLDER + "/" + h + "/" + filename ):
        return send_file( UPLOAD_FOLDER + "/" + h + "/" + filename, mimetype = 'image/png' )
    else:
        return send_file( "./app/not_avaiable.png", mimetype = 'image/png' )

@api.route( '/uploadfile' )
def upload_interface():
    return render_template( "uploadfile.html" )

@api.route( '/<uuid>' )
def mainui( uuid ):
    if os.path.isdir( UPLOAD_FOLDER + "/" + uuid ):
        lst = os.listdir( UPLOAD_FOLDER + "/" + uuid )
        name = uuid
        filename = None
        for f in lst:
            n, ext = os.path.splitext( f )
            if ext in ALLOWED_EXTENSIONS:
                name = n
                filename = f
        
        return render_template( 
            "ui.html",
            uuid = uuid,
            name = name,
            filename = filename
        )
    
    else:
        abort( 404 )

@api.route( '/<uuid>.zip' )
def get_archive( uuid ):
    io = StringIO()
    
    filename = uuid
    
    with zipfile.ZipFile( io, 'w', zipfile.ZIP_DEFLATED ) as fp:
        for root, dirs, files in os.walk( UPLOAD_FOLDER + "/" + uuid ):
            for file in files:
                name, ext = os.path.splitext( file )
                if ext in ALLOWED_EXTENSIONS:
                    filename = name
                
                if "_preview" in file:
                    continue
                else:
                    fp.write( os.path.join( root, file ), file )
    
    io.seek( 0 )
    
    return send_file( io, attachment_filename = filename + ".zip", as_attachment = True )
        
################################################################################
#    Main runner

if __name__ == '__main__':
    if not os.path.isdir( UPLOAD_FOLDER ):
        os.makedirs( UPLOAD_FOLDER )
    
    api.run( debug = True, host = "0.0.0.0", threaded = True )
